@@ -3,10 +3,17 @@ package org.gcube.portal.istischeduler.impl;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
+import java.sql.Connection;
+import java.sql.DriverManager;
+import java.sql.PreparedStatement;
+import java.sql.ResultSet;
+import java.sql.SQLException;
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Locale;
 import java.util.Properties;
+import java.util.Set;
 
 import org.gcube.portal.istischeduler.PortalUserManager;
 import org.gcube.portal.istischeduler.ex.UserManagementSystemException;
@@ -39,7 +46,7 @@ public class LiferayPortalUserManager implements PortalUserManager {
 	 * logger
 	 */
 	private static final Logger _log = LoggerFactory.getLogger(LiferaySiManager.class);
-	
+
 	private static final String ROOT_ORG = "rootorganization";
 	private static final String DEFAULT_COMMUNITY_PROP_NAME = "defaultcommunity";
 	private static final String DEFAULT_COMPANY_WEB_ID = "liferay.com";
@@ -51,14 +58,18 @@ public class LiferayPortalUserManager implements PortalUserManager {
 	private final static String FISCAL_CODE = "FISCAL_CODE";
 	private final static String OPTIONAL_IDENTIFIER = "OPTIONAL_IDENTIFIER";
 
-	public LiferayPortalUserManager() {
-		super();		
+	private String administratorUsername;
+	/**
+	 * @param administratorUsername one of the administrators username (screenname) of the portal
+	 */
+	public LiferayPortalUserManager(String administratorUsername) {
+		this.administratorUsername = administratorUsername;
 	}
 	/**
 	 * {@inheritDoc}
 	 */
 	@Override
-	public boolean createUser(String administratorUsername, PortalUserModel user2Add) throws UserManagementSystemException, UserRetrievalFault {
+	public boolean createUser(PortalUserModel user2Add) throws UserManagementSystemException, UserRetrievalFault {
 		_log.debug("Creating user with username = " + user2Add.getUsername());
 		try {
 			long companyId = getCompany().getCompanyId();
@@ -71,28 +82,28 @@ public class LiferayPortalUserManager implements PortalUserManager {
 			Locale locale = new Locale("en_US");
 			ServiceContext serviceContext = new ServiceContext();
 			String password2 = null;		
-	
+
 			Group defaultCommunity = GroupLocalServiceUtil.getGroup(companyId, getDefaultCommunityName());
 			long[] groupids = {defaultCommunity.getGroupId()};
-			
+
 			User toAdd = UserLocalServiceUtil.addUser(
 					0L, companyId, true, password1, password2, false, 
 					user2Add.getUsername(), user2Add.getEmail(), 0L,"",
-					locale, user2Add.getFirstname(), "mn", user2Add.getLastname(), 0, 0, true, 
+					locale, user2Add.getFirstname(), "", user2Add.getLastname(), 0, 0, true, 
 					1, 1, 1940, "", groupids, null, null, null, true, serviceContext);
-			
-			
+
+
 			//add the custom attrs
 			if (! toAdd.getExpandoBridge().hasAttribute(POSITION)) 	toAdd.getExpandoBridge().addAttribute(POSITION);
 			if (! toAdd.getExpandoBridge().hasAttribute(LABS)) 	toAdd.getExpandoBridge().addAttribute(LABS);
 			if (! toAdd.getExpandoBridge().hasAttribute(FISCAL_CODE)) toAdd.getExpandoBridge().addAttribute(FISCAL_CODE);
 			if (! toAdd.getExpandoBridge().hasAttribute(OPTIONAL_IDENTIFIER)) toAdd.getExpandoBridge().addAttribute(OPTIONAL_IDENTIFIER);
-			
+
 			toAdd.getExpandoBridge().setAttribute(POSITION, user2Add.getPosition());
 			toAdd.getExpandoBridge().setAttribute(LABS, user2Add.getGroups());
 			toAdd.getExpandoBridge().setAttribute(FISCAL_CODE, user2Add.getFiscalCode());
 			toAdd.getExpandoBridge().setAttribute(OPTIONAL_IDENTIFIER, user2Add.getOptionalIdentifier());
-					
+
 		} catch (SystemException e) {
 			throw new UserManagementSystemException("Error creating user ",user2Add.getUsername(), e);
 		} catch (PortalException e) {			
@@ -113,26 +124,49 @@ public class LiferayPortalUserManager implements PortalUserManager {
 			User user = null;
 			if (user2Update.getUsername() == null || user2Update.getUsername().isEmpty())
 				return false;
-			try {
-				long companyId = getCompany().getCompanyId();
-				user = UserLocalServiceUtil.getUserByScreenName(companyId, user2Update.getUsername());
-				user.setEmailAddress(user2Update.getEmail());
-				user.setFirstName(user2Update.getFirstname());
-				user.setLastName(user2Update.getLastname());
-				user.getExpandoBridge().setAttribute(FISCAL_CODE, user2Update.getFiscalCode());
-				user.getExpandoBridge().setAttribute(OPTIONAL_IDENTIFIER, user2Update.getOptionalIdentifier());
-			}
-			catch (com.liferay.portal.NoSuchUserException es) {
-				_log.warn("User to modify does not exist");
-				return false;
-			}
-			catch (PortalException e) {
-				throw new UserRetrievalFault("Error retrieving user by screen name ",user2Update.toString(), e);
-			}
+
+			long companyId = getCompany().getCompanyId();
+			User user4Permissions = UserLocalServiceUtil.getUserByScreenName(companyId, administratorUsername);
+			PermissionChecker permissionChecker = PermissionCheckerFactoryUtil.create(user4Permissions, false);
+			PermissionThreadLocal.setPermissionChecker(permissionChecker);
 			
-			UserLocalServiceUtil.updateUser(user);
-		} catch (SystemException e) {
-			throw new UserManagementSystemException("Error updating user  ",user2Update.toString(), e);
+			user = UserLocalServiceUtil.getUserByScreenName(companyId, user2Update.getUsername());
+			
+			System.out.println("updateUser param = "+user2Update.getUsername());
+			
+			if (user2Update.getEmail() != null && user2Update.getEmail().compareTo("") != 0)
+				user.setEmailAddress(user2Update.getEmail());
+			
+			if (user2Update.getFirstname() != null && user2Update.getFirstname().compareTo("") != 0)
+				user.setFirstName(user2Update.getFirstname());
+			
+			if (user2Update.getLastname() != null && user2Update.getLastname().compareTo("") != 0)
+				user.setLastName(user2Update.getLastname());
+			
+			if (user2Update.getPosition() != null && user2Update.getPosition().compareTo("") != 0)
+				user.getExpandoBridge().setAttribute(POSITION, user2Update.getPosition());
+			
+			if (user2Update.getGroups() != null && user2Update.getGroups().compareTo("") != 0)
+				user.getExpandoBridge().setAttribute(LABS, user2Update.getGroups());
+			
+			if (user2Update.getFiscalCode() != null && user2Update.getFiscalCode().compareTo("") != 0)
+				user.getExpandoBridge().setAttribute(FISCAL_CODE, user2Update.getFiscalCode());
+			
+			if (user2Update.getOptionalIdentifier() != null && user2Update.getOptionalIdentifier().compareTo("") != 0)
+				user.getExpandoBridge().setAttribute(OPTIONAL_IDENTIFIER, user2Update.getOptionalIdentifier());		
+			
+			UserLocalServiceUtil.updateUser(user);			
+			System.out.println("User and Custom Attributes updated: " + user.getScreenName());
+		}
+		catch (com.liferay.portal.NoSuchUserException es) {
+			_log.warn("User to modify does not exist");
+			return false;
+		}
+		catch (PortalException e) {
+			throw new UserRetrievalFault("Error retrieving user by screen name ",user2Update.toString(), e);
+		}
+		catch (Exception ex) {
+			ex.printStackTrace();
 		}
 		return true;
 	}
@@ -146,6 +180,10 @@ public class LiferayPortalUserManager implements PortalUserManager {
 		PortalUserModel um = null;
 		try {
 			long companyId = getCompany().getCompanyId();
+			user = UserLocalServiceUtil.getUserByScreenName(companyId, administratorUsername);
+			PermissionChecker permissionChecker = PermissionCheckerFactoryUtil.create(user, false);
+			PermissionThreadLocal.setPermissionChecker(permissionChecker);
+
 			user = UserLocalServiceUtil.getUserByScreenName(companyId, username);
 			String position = (user.getExpandoBridge().getAttribute(POSITION) != null) ? user.getExpandoBridge().getAttribute(POSITION).toString() : "";
 			String labs =  (user.getExpandoBridge().getAttribute(LABS) != null) ? user.getExpandoBridge().getAttribute(LABS).toString() : "";
@@ -163,8 +201,8 @@ public class LiferayPortalUserManager implements PortalUserManager {
 					);
 		} catch (SystemException e) {
 			throw new UserManagementSystemException("Error retrieving user by screename  ", username, e);
-		} catch (PortalException e) {
-			throw new UserManagementSystemException("Error retrieving user by screename  ", username, e);
+		} catch (Exception e) {
+			e.printStackTrace();
 		}
 		return um;
 	}
@@ -188,13 +226,20 @@ public class LiferayPortalUserManager implements PortalUserManager {
 		List<User> users = null;
 		List<PortalUserModel> userModels = new ArrayList<PortalUserModel>();
 		try {
+			long companyId = getCompany().getCompanyId();
+
+
 			users = UserLocalServiceUtil.getUsers(0, UserLocalServiceUtil.getUsersCount());
 			for(User user : users){
+				user = UserLocalServiceUtil.getUserByScreenName(companyId, administratorUsername);
+				PermissionChecker permissionChecker = PermissionCheckerFactoryUtil.create(user, false);
+				PermissionThreadLocal.setPermissionChecker(permissionChecker);
+
 				String position = (user.getExpandoBridge().getAttribute(POSITION) != null) ? user.getExpandoBridge().getAttribute(POSITION).toString() : "";
 				String labs =  (user.getExpandoBridge().getAttribute(LABS) != null) ? user.getExpandoBridge().getAttribute(LABS).toString() : "";
 				String fiscalCode = (user.getExpandoBridge().getAttribute(FISCAL_CODE) != null) ? user.getExpandoBridge().getAttribute(FISCAL_CODE).toString() : "";
 				String optionalId = (user.getExpandoBridge().getAttribute(OPTIONAL_IDENTIFIER) != null) ? user.getExpandoBridge().getAttribute(OPTIONAL_IDENTIFIER).toString() : "";
-			
+
 				PortalUserModel um = new PortalUserModel(
 						user.getScreenName(), 
 						user.getFirstName(), 
@@ -209,6 +254,10 @@ public class LiferayPortalUserManager implements PortalUserManager {
 			}
 		}catch (SystemException e) {
 			throw new UserManagementSystemException("Error retrieving list of users ", e);
+		} catch (PortalException e) {
+			e.printStackTrace();
+		} catch (Exception e) {
+			e.printStackTrace();
 		}
 		return userModels;
 	}
@@ -286,4 +335,50 @@ public class LiferayPortalUserManager implements PortalUserManager {
 		_log.debug("Returning Default Community Name: " + toReturn );
 		return toReturn;
 	}
+
+	public void updateGroups() throws Exception {
+		_log.info("updateGroups called");
+		String dbUrl = "jdbc:postgresql://" + LiferaySiManager.getSiTestEndPoint().getHostName()+":"+LiferaySiManager.getSiTestEndPoint().getPort()+"/"+LiferaySiManager.getSiTestEndPoint().getDbName();
+		String dbUser = LiferaySiManager.getSiTestEndPoint().getUser();
+		String dbPassword = LiferaySiManager.getSiTestEndPoint().getPassword();
+		Connection con = null;
+		try {
+			List<PortalUserModel> currUsers = listPortalUsers();
+			_log.info("currUsers size = "  + currUsers.size());
+
+			con = DriverManager.getConnection(dbUrl, dbUser, dbPassword);
+			_log.info("getConnection to SI OK");
+			String select = 
+					"SELECT anagrafico.codicefiscale, anagrafico.cognome, anagrafico.nome, gruppo, gruppo_pers.al "
+							+ "FROM gruppo_pers, anagrafico "
+							+ "WHERE gruppo_pers.cf = anagrafico.codicefiscale and anagrafico.codicefiscale = ? "
+							+ "GROUP BY anagrafico.cognome, anagrafico.nome, gruppo_pers.gruppo, anagrafico.codicefiscale, gruppo_pers.al";
+
+			PreparedStatement statement = con.prepareStatement(select);
+
+			for (PortalUserModel user : currUsers) {
+				if (user.getFiscalCode() != null) {
+					statement.setString(1, user.getFiscalCode());
+					ResultSet rs = statement.executeQuery();
+					Set<String> set = new HashSet<String>();
+					//add the groups to the set
+					while (rs.next()) {
+						set.add(rs.getString(4));				
+					}
+					String groups = set.toString();
+					groups = groups.substring(1, groups.length()-1);
+					user.setGroups(groups);
+					boolean result = updateUser(user);
+					_log.info("Update user " + user.getLastname() + "=" + result);
+				}
+			}
+			con.close();
+			LiferaySiManager.log2SI("anagrafico,gruppo_pers");
+			_log.info("log2SI = OK");
+		} catch (SQLException e) {
+			e.printStackTrace();
+		} finally {
+			con.close();
+		}
+	}
 }
